/*
   This program changes the view of the camera
   from looking out at the horizon to looking
   straight down at the origin.
*/
import java.awt.Color;

public class Horizon
{
   public static void main(String[] args)
   {
      int k = 0; // number of triangles in the triangle fans at the top and bottom
      int n = 0; // number of triangle strips between the top and bottom

      // Check for two command line arguments.
      if (args.length >=2)
      {
         k = Integer.parseInt( args[0] );
         n = Integer.parseInt( args[1] );
      }
      else
      {
         System.err.printf("Usage: %s <k> <n>", Horizon.class.getName());
         System.exit(-1);;
      }

      // Create several Model objects.
      Model m1 = new CubeModel(1, 1, 1);
      for (Triangle t : m1.triangleList)
      {
         t.v[0].setColorRandom();
         t.setColor( t.v[0].getColor() );
      }
      m1.modelTranslate(-4, 0, 0);

      Model m2 = new SphereModel(k, n, 1);
      for (Triangle t : m2.triangleList)
      {
         t.v[0].setColorRandom();
         t.setColor( t.v[0].getColor() );
      }
      m2.modelTranslate(-2, 3, 0);

      Model m3 = new ConeModel(4, 4, 1, 2);
      for (Triangle t : m3.triangleList)
      {
         t.v[0].setColorRandom();
         t.setColor( t.v[0].getColor() );
      }

      Model m4 = new CylinderModel(k, n, 1, 4);
      for (Triangle t : m4.triangleList)
      {
         t.v[0].setColorRandom();
         t.setColor( t.v[0].getColor() );
      }
      m4.modelTranslate(3, 0, 0);

      Model m5 = new PanelModel(-25, 25, -25, 5, Color.white, Color.black);
      m5.doLighting = false;

      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the models to the Scene.
      scene.modelList.add(m1);
      scene.modelList.add(m2);
      scene.modelList.add(m3);
      scene.modelList.add(m4);
      scene.modelList.add(m5);

      // Give the Scene a view matrix.
      scene.view2Identity();

      // Give the scene a projection matrix.
      scene.proj2Identity();

      // Set up the scene's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 200.0;
      scene.projPerspective(fov, aspect, near, far);

      // Set up the scene's "camera".
      double eyex=0,    eyey=10,    eyez=0;      // where the camera is
      double centerx=0, centery=0, centerz=-25;  // where the camera is pointed
      double upx=0,     upy=1,     upz=-1;       // where the "top" of the camera is pointed
      scene.viewLookAt(eyex,    eyey,    eyez,      // What happens if the up vector is (0,1,0)?
                       centerx, centery, centerz,
                       upx,     upy,     upz);


      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.darkGray);
      // Render our scene into the frame buffer.
      SceneRender.renderScene(scene, fb);
      // Save the resulting image in a file.
      fb.dumpFB2File( String.format("Horizon_Frame%02d.ppm", 0) );

      // Change where the camera is pointing.
      for (int i = 1; i <= 50; i++)
      {
         // Change the view.
         scene.view2Identity();
         eyex=0;    eyey=10;    eyez=0;
         centerx=0; centery=0; centerz=-25+(i*0.5); // change where the camera is pointed
         upx=0;     upy=1;     upz=-1;
         scene.viewLookAt(eyex,    eyey,    eyez,
                          centerx, centery, centerz,
                          upx,     upy,     upz);
         // Render again.
         fb.clearFB(java.awt.Color.darkGray);
         SceneRender.renderScene(scene, fb);
         fb.dumpFB2File(String.format("Horizon_Frame%02d.ppm", i));
      }
   }
}